# -*- coding: utf-8 -*-

from django.test import TestCase
from django.utils import timezone

from django_pg_current_timestamp.test_app.models import TestModel
from django_pg_current_timestamp import CurrentTimestamp


class CurrentTimestampTest(TestCase):
    initiated = False

    def testCurrentTimestamp(self):
        a = TestModel.objects.create(value='altavista.com')
        b = TestModel.objects.get(id=a.id)

        # These shouldn't match, because the TS returned with the initial object is app-generated, whereas the queried object `b` should have the databases `CURRENT_TIMESTAMP` value.
        self.assertNotEqual(a.created_ts, b.created_ts)
        self.assertNotEqual(a.modified_ts, b.modified_ts)

        # These should match if the object was created using `CURRENT_TIMESTAMP`.
        self.assertEqual(b.created_ts, b.modified_ts)
        self.assertEqual(b.created_ts, b.modified_ts)

        b.value='scala.sh'
        b.save()
        c = TestModel.objects.get(id=a.id)

        self.assertNotEqual(b.modified_ts, c.modified_ts)
        self.assertNotEqual(b.modified_ts, c.modified_ts)




# -*- coding: utf-8 -*-

"""Adds django compatibility with / capability to use `CURRENT_TIMESTAMP` on DateTimeField objects."""

import logging

from django.utils import timezone

from psycopg2.extensions import ISQLQuote


__version__ = '0.2.1'
__author__ = 'Jay Taylor [@jtaylor]'


logger = logging.getLogger(__name__)


class CurrentTimestamp(object):

    def __str__(self):
        return 'CURRENT_TIMESTAMP'

    def as_sql(self, qn, val):
        return self.__str__(), {}

    def __conform__(self, proto):
        """Does the given protocol conform to what Psycopg2 expects?"""
        if proto == ISQLQuote:
            return self
        else:
            raise Exception('Error implementing psycopg2 protocol. Is psycopg2 installed?')

    def getquoted(self):
        """Returns a properly quoted string for use in PostgreSQL/PostGIS."""
        # Psycopg will figure out whether to use E'\\000' or '\000'.
        return self.__str__()

    @classmethod
    def patch(cls, field):
        orig_pre_save = field.pre_save
        orig_prep_db = field.get_db_prep_value
        orig_prep_lookup = field.get_prep_lookup
        orig_db_prep_lookup = field.get_db_prep_lookup

        def pre_save(self, model_instance, add):
            """
            Pre-save `CURRENT_TIMESTAMP` injector.
            NB: The returned value is what will go into the database, and the `timezone.now()` value will be set on the model instance.
            """
            if self.auto_now or (self.auto_now_add and add):
                value = CurrentTimestamp()
                setattr(model_instance, self.attname, timezone.now()) # Attach an approximate TS to the object.
                return value
            else:
                return orig_pre_save(self, model_instance, add)

        def prep_db_value(self, value, connection, prepared=False):
            result = value if isinstance(value, cls) else orig_prep_db(self, value, connection, prepared)
            #logger.debug('prep_db_value :: name={} type(result)={} result={}'.format(self.name, type(result), result))
            return result

        def prep_lookup(self, lookup_type, value):
            result = value if isinstance(value, cls) else orig_prep_lookup(self, lookup_type, value)
            #logger.debug('prep_lookup :: name={} type(result)={} result={}'.format(self.name, type(result), result))
            return result

        def prep_db_lookup(self, lookup_type, value, connection, prepared=True):
            result = value if isinstance(value, cls) else orig_db_prep_lookup(self, lookup_type, value, connection=connection, prepared=True)
            #logger.debug('prep_db_lookup :: name={} type(result)={} result={}'.format(self.name, type(result), result))
            return result

        field.pre_save = pre_save
        field.get_db_prep_value = prep_db_value
        field.get_prep_lookup = prep_lookup
        field.get_db_prep_lookup = prep_db_lookup


def init():
    """Activation for automatic support of DateTimeField fields with `auto_now` and/or `auto_now_add` columns."""
    from django.db.models import DateTimeField
    logger.info('django_pg_current_timestamp :: Monkey-patching django.db.models.DateTimeField to enable automatic `CURRENT_TIMESTAMP` support for DateTimeField')
    CurrentTimestamp.patch(DateTimeField)




# -*- coding: utf-8 -*-

from django.db import models

class TestModel(models.Model):
	value = models.CharField(max_length=100)
	created_ts = models.DateTimeField(auto_now_add=True, null=False, blank=False)
	modified_ts = models.DateTimeField(auto_now=True, null=False, blank=False)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from distutils.core import setup
from setuptools import find_packages

import django_pg_current_timestamp

setup(
    name='django-pg-current-timestamp',
    version=django_pg_current_timestamp.__version__,
    author='Jay Taylor',
    author_email='jay@jaytaylor.com',
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/threatstream/django-pg-current-timestamp',
    license=open(os.path.join(os.path.dirname(__file__), 'LICENSE'), 'r').read(),
    description='Add true postgresql `CURRENT_TIMESTAMP` support to Django + PostgreSQL',
    long_description='Documentation is available on github: https://github.com/threatstream/django-pg-current-timestamp',
    install_requires=open(os.path.join(os.path.dirname(__file__), 'requirements.txt'), 'r').read().strip().split('\n'),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    test_suite='runtests.runtests',
    zip_safe=False,
)

